/**
 * Informatique Industrielle et Java
 * Projet exceptions du chapitre2 
 *
 * @author F. Mallet et F. Boeri
 * @version 1
 */
public class CompteurNBits {
    /** repr?sente la valeur courante du compteur 
      * 0 <= etat < 2^n 
      */
    private int etat = 0;
    
    /** repr?sente le nombre de bits utilis?s pour coder le compteur, 
      * le compteur a 2^n ?tats,  0<n<=30 */
    private byte n;
        
    /** constructeur qui initialise le nombre de bit utilis?s 
     * @param valeur nombre de bits du compteur, 0<valeur<=30
     */
    public CompteurNBits(byte n) {
         if (n<=0 || n>30) {
              throw new RuntimeException("n doit etre compris entre 0 et 30");
        }
       this.n = n;
    }
    
    /** Constructeur qui inialise l'etat et le nombre de bits utilis?s
     * @param etat defini l'etat initial du compteur, 0 <= etat < 2^n
     * @param n nombre de bits du compteur, 0 < n <= 30
     */
    public CompteurNBits(int etat, byte n) {
        this(n);
        if (etat<0 || (etat >= (1<<n))) {
            throw new RuntimeException("etat est negatif ou superieur a 2^n");
        }
        this.etat = etat;
    }
    
    /**
     * @return la valeur de l'?tat courant du compteur 
     */
    public int getEtat() { return etat; }
    
    /** Incr?mente l'?tat du compteur */
    public void incrementer() {
        if (etat == ((1<<n) - 1)) {
            etat = 0;
        } else {
            etat += 1;
        }
    }
    
    /** incr?mente le compteur d'un nombre arbitraire d'?tats
     * @param nombre nombre d'?tat ? incr?menter, 0 <= nombre < 2^n
     */
    public void incrementer ( int nombre ) {
    // si on est s?r de ne pas d?passer l'?tat limite
        if ( etat < ((1<<n) - nombre)) { 
            etat += nombre;                  // l'?tat est incr?ment?
        } else {                        // sinon
            etat += (nombre - (1<<n));  // calcul le nouvel ?tat
        }
    }
}